!debug = 1

include ".\fanchartlib.prg"

if !debug = 0 then
     logmode +addin
endif

'store current smpl
%smpl = @pagesmpl

'Workfile Validity Checking
if @pagefreq="U" then
     %msg = "Please open a time-series workfile"
     @uiprompt(%msg)
     stop
endif

if @ispanel=1 then
     %msg = "Panel workfiles not allowed"
     @uiprompt(%msg)
     stop
endif

'Variable declarations / options
%modeseries = %0
%uncertseries = %1
%skewseries = %2
%title = "Fan chart of forecasts of " + _this.@displayname
if !debug =1 then
     %modeseries = "mode"
     %uncertseries = "uncertainty"
     %skewseries = "skewness"
endif

%probs = "0.025 0.15 0.35 "
%chartcolor = "Red"
!include50th = 1
%outputsername = ""
%horizline = "2"

!dogui = @len(%modeseries)=0 and @len(%uncertseries)=0 and @len(%skewseries)=0
if !debug = 1 then
     !dogui = 1
endif

'GUI
if !dogui then
     !result = 1
     while !result = 1
          !result = @uidialog("caption", "Bank of England Fan Charts", _
                     "edit", %modeseries, "Mode series", _
                     "edit", %uncertseries, "Uncertainty series", _
                     "edit", %skewseries, "Skewness series", _
                     "edit", %title, "Chart title", _
                     "button", "Adv. Options" _
                     )
          if !result = 1 then
                %coloropts = "Red Green Blue Yellow"
                !result2 = @uidialog("caption", "Advanced Options", _
                          "edit", %probs, "Probability values (one sided)", _
                          "list", %chartcolor, "Chart base color", %coloropts, _
                          "check", !include50th, "Include median line", _
                          "edit", %horizline, "Add horizonal line at value (optional)", _
                          "edit", %outputsername, "Output series pattern (optional)" _
                          )
          endif
     wend
     if !result = -1 then
          stop
     endif
endif  


'check probs are valid
for !i=1 to @wcount(%probs)
     %p = @word(%probs,!i)
     !invalid = 0
     if @val(%p)=NA then
          !invalid = 1
     endif
     if @val(%p)<0 or @val(%p)>=0.5 then
          !invalid=1
     endif
     if !invalid=1 then
          %msg = %p + " is not a valid one sided probability (must be 0>p<0.5)"
          @uiprompt(%msg)
          stop
     endif
next

!np = @wcount(%probs)*2 + !include50th


'make vector of probabilities
%pname = @getnextname("_probs")
vector(!np) {%pname}
!j=1
for !i=1 to @wcount(%probs)
     {%pname}(!j)=@val(@word(%probs,!i))
     {%pname}(!j+1) = 1-{%pname}(!j)
     !j=!j+2
next
'add in median
if !include50th=1 then
     {%pname}(!np)=0.5
endif

'storage for quantiles
%qname = @getnextname("_quants")
vector(!np) {%qname}


'calculate quantiles
smpl if {%modeseries}<>na
!t = @obssmpl
!offset = @ifirst({%modeseries})
%matname = @getnextname("_mat")
matrix(!t,!np) {%matname}
for !i=!offset to !offset+!t-1
     !mode = {%modeseries}(!i)
     !skew = {%skewseries}(!i)
     !uncert = {%uncertseries}(!i)
     call qsplitnorm(!mode, !uncert, !skew, {%pname}, {%qname}) 
     rowplace({%matname}, @transpose({%qname}), !i-!offset+1)
next

'convert to group
%grpname = @getnextname("_grp")

if @len(%outputsername)>0 then
     group {%grpname}
     for !i=1 to !np
          %suffix = @str(@round({%pname}(!i)*1000))
          series {%outputsername}_{%suffix}
          {%grpname}.add {%outputsername}_{%suffix}
     next
endif

smpl if {%modeseries}<>na
mtos({%matname}, {%grpname})

'fill in first date
%first = {%modeseries}.@first
for !i=1 to !np
     smpl {%first}-1 {%first}-1
     {%grpname}(!i) = _this
next


'add cpi to the start of the group
group {%grpname} _this {%grpname}
smpl 2011 @last-1
%grphname = @getnextname("_graph")

call MakeFanChart({%grpname}, %chartcolor, %grphname, !include50th, %title, @val(%horizline))
_this.display {%grphname}

'cleanup
smpl {%smpl}
%thisname = _this.@name
{%grpname}.drop {%thisname}
%sernames = {%grpname}.@members
if @len(%outputsername)=0 then
     d(noerr) {%sernames}
endif

d(noerr)  {%pname} {%qname} {%grpname} {%matname}
d(noerr) {%grphname}







'
' -1 < g < 1
'
subroutine local qsplitnorm(scalar m, scalar sigma, scalar g, vector p, vector q)

scalar s1 = sigma / @sqrt(1 - g)
scalar s2 = sigma / @sqrt(1 + g)
scalar pbal = s1 / (s1 + s2)
scalar pi = 2*@asin(1)
scalar sqrt2pi = @sqrt(2*pi)
scalar cscale = @sqrt(2/pi) / (s1 + s2)

scalar alpha
scalar k
for !i = 1 to @rows(p)
     alpha = p(!i)
     if (alpha <= pbal) then
         k = alpha / (cscale * sqrt2pi * s1)
          q(!i) = m + s1 * @qnorm(k)
     else
          k = (alpha + cscale * sqrt2pi * s2 - 1) / (cscale * sqrt2pi * s2)
          q(!i) = m + s2 * @qnorm(k)
     endif
next


endsub


