
subroutine MakeFanChart(group grp, string %color, string %name, scalar includeMedian, string %title, scalar lineVal)
	!serCount = grp.@count
	
	' create the list of series that will be band graphs
	for !i=2 to !serCount-includeMedian
		%bandmembers = 	%bandmembers + @str(!i)
		
		if !i<>!serCount-includeMedian then
			%bandmembers = 	%bandmembers +  ","			
		endif
	next
	
	' create the graph
	if includeMedian=1 then
		freeze({%name}) grp.mixed line(1,!serCount) band({%bandmembers})
	else
		freeze({%name}) grp.mixed line(1) band({%bandmembers})	
	endif
		
	'set fan colors
	if includeMedian=1 then
		!numFans = (!serCount-2)/2
	else
		!numFans = (!serCount-1)/2
	endif
		
	%colorvec = @getnextname("_rgbColor")
	vector(3) {%colorvec }

	!colorset = 0	
	if %color="Red" then
		{%colorvec}(1) = 255
		{%colorvec}(2) = 0
		{%colorvec}(3) = 0
		!colorset = 1
	endif
	if %color="Green" then
		{%colorvec}(1) = 80
		{%colorvec}(2) = 155
		{%colorvec}(3) = 80
		!colorset = 1
	endif
	if %color="Blue" then
		{%colorvec}(1) = 0
		{%colorvec}(2) = 0
		{%colorvec}(3) = 200
		!colorset = 1
	endif	
	if %color="Yellow" then
		{%colorvec}(1) = 190
		{%colorvec}(2) = 190
		{%colorvec}(3) = 40
		!colorset = 1
	endif	
	
	if !colorset=0 then
		seterr Invalid color specified		
		return
	endif
	
	!hue=0
	!lum=0
	!sat=0
		
	call RGBToHLS({%colorvec }(1), {%colorvec }(2), {%colorvec }(3), !hue, !lum, !sat)	
	
	'calculate shade increments in percent where 220 is the max luminance
	!shadeInc = (220-!hue)/(!numFans*240)
	
	%ltcolorvec = @getnextname("_lightColor")
	vector(3) {%ltcolorvec }

 	!pctShade = 1+200/240	 + !shadeInc		'max shading
 	
 	'' set the fan colors
	for !i=1 to !numFans
		!pctShade = !pctShade - !shadeInc
		call lightenRGB({%colorvec }(1), {%colorvec }(2), {%colorvec }(3), !pctShade ,  {%ltcolorvec })
		
		!R = {%ltcolorvec }(1)
		!G = {%ltcolorvec }(2)
		!B = {%ltcolorvec }(3)
		{%name}.setelem(!i) fillcolor(@rgb(!R, !G, !B))
	next
	
	'set line colors
	!R = {%colorvec }(1)
	!G = {%colorvec }(2)
	!B = {%colorvec }(3)
	{%name}.setelem(1) linecolor(@rgb(!R, !G, !B)) linewidth(2)
	if includeMedian= 1 then 
     		{%name}.setelem(2) linecolor(@rgb(!R, !G, !B)) linewidth(2)
	endif 	
	
	'assign all series to the right axis
	for !i=1 to !serCount-1
		{%name}.setelem(!i) axis(r)
	next
	
	if includeMedian = 1 then 
     		{%name}.setelem(!serCount) axis(r)
	endif		
	
'	{%name}.axis(r) range(-3,6)
	{%name}.options frameaxes(all)
	{%name}.draw(l,right, width(1.5), top) lineval

	' add shade object
	%forecastSeries = grp.@seriesname(2)
	!forecastStartObs = @ifirst({%forecastSeries })
	%forecastStartDate = @otod(!forecastStartObs)
	
	!forecastEndObs =  @ilast({%forecastSeries })
	%forecastEndDate = @otod(!forecastEndObs)
	{%name}.draw(s, b, color(239,239,239)) {%forecastStartDate} {%forecastEndDate}
	
	'add title
	{%name}.addtext(t) %title		
	
	'turn off legend
	{%name}.options -legend	
	
	' put ticks on the inside
	{%name}.axis(r) ticksin
	{%name}.axis(b) ticksin
	
	'change date format
	{%name}.datelabel format("YYYY")
	
	' cleanup
	d(noerr) {%ltcolorvec } {%colorvec } 
endsub

'lighten or darken the passed in color and returns the rgb values into the passed in vector
subroutine local lightenRGB(scalar R, scalar G, scalar B, scalar pct, vector outvec)
	scalar H
	scalar L
	scalar S
		
	call RGBToHLS(R, G, B, H, L, S)
	
	scalar postR
	scalar postG
	scalar postB
	
	call HLStoRGB(H, L*pct, S, postR, postG, postB )
	
	outvec(1) = postR
	outvec(2) = postG 
	outvec(3) = postB
	
endsub

' find the max of 2 scalars
subroutine max(scalar x, scalar y, scalar out)
	if x>y then
		out = x
	else
		out = y
	endif	
endsub

' find the max of 3 scalars
subroutine local max3(scalar x, scalar y, scalar z, scalar out)
	scalar a
	
	call max(x, y, a)
	
	call max( z, a, out)
endsub

' find the min of 2 scalars
subroutine min(scalar x, scalar y, scalar out)
	if x<y then
		out = x
	else
		out = y
	endif	
endsub

' find the max of 3 scalars
subroutine local min3(scalar x, scalar y, scalar z, scalar out)
	scalar a
	
	call min(x, y, a)
	
	call min( z, a, out)
endsub

subroutine local  RGBToHLS(scalar R, scalar G, scalar B, scalar H, scalar L, scalar S)
	!HLSMAX = 240
	!RGBMAX = 255
    	!UNDEFINED=  (!HLSMAX*2/3)
    	
	' calculate lightness
	!cMax = R
	!cMin = R
	call max3(R, G ,B, !cMax)
	call min3(R, G ,B, !cMin)
	
	L = ( ((!cMax+!cMin)*!HLSMAX) + !RGBMAX )/(2*!RGBMAX)

	if  !cMax = !cMin then         ' r=g=b --> achromatic case
		S = 0
		' saturation
		H = !UNDEFINED             ' hue
	else                        ' chromatic case
		' saturation
		if (L <= (!HLSMAX/2)) then
			S = ( ((!cMax-!cMin)*!HLSMAX) + ((!cMax+!cMin)/2) ) / (!cMax+!cMin)
		else
			S = ( ((!cMax-!cMin)*!HLSMAX) + ((2*!RGBMAX-!cMax-!cMin)/2) )/ (2*!RGBMAX-!cMax-!cMin)
		endif
		
		' hue
		!Rdelta = ( ((!cMax-R)*(!HLSMAX/6)) + ((!cMax-!cMin)/2) ) / (!cMax-!cMin)
		!Gdelta = ( ((!cMax-G)*(!HLSMAX/6)) + ((!cMax-!cMin)/2) ) / (!cMax-!cMin)
		!Bdelta = ( ((!cMax-B)*(!HLSMAX/6)) + ((!cMax-!cMin)/2) ) / (!cMax-!cMin)


		if (R = !cMax) then         
			H = !Bdelta - !Gdelta
		else
			if (G = !cMax) then
				H = (!HLSMAX/3) + !Rdelta - !Bdelta
			else ' B == cMax
				H = ((2*!HLSMAX)/3) + !Gdelta - !Rdelta
			endif
		endif

		if (H < 0) then         
			H = H + !HLSMAX
		endif
		if ( H > !HLSMAX) then
			H = H - !HLSMAX
		endif 
	endif
endsub


' utility routine for HLStoRGB
subroutine local HueToRGB(scalar n1, scalar n2, scalar hue, scalar X) 
	!HLSMAX = 240
	!RGBMAX = 255
    	!UNDEFINED=  (!HLSMAX*2/3)

	' range check: note values passed add/subtract thirds of range
	if (hue < 0) then 
		hue = hue + !HLSMAX
	endif

	if (hue > !HLSMAX) then
		hue = hue - !HLSMAX
	endif

	' return r,g, or b value from this tridrant
	if (hue < (!HLSMAX/6)) then
		x = ( n1 + (((n2-n1)*hue+(!HLSMAX/12))/(!HLSMAX/6)) )
		return
	endif
	
	if (hue < (!HLSMAX/2)) then
		x =  ( n2 )
		return
	endif
	
	if (hue < ((!HLSMAX*2)/3)) then
		x =  ( n1 + (((n2-n1)*(((!HLSMAX*2)/3)-hue)+(!HLSMAX/12))/(!HLSMAX/6)))
		return
	else
		x = ( n1 )
		return
	endif
	
endsub


subroutine local HLStoRGB(scalar hue, scalar lum, scalar sat, scalar R, scalar G, scalar B )
	!HLSMAX = 240
	!RGBMAX = 255
    	!UNDEFINED=  (!HLSMAX*2/3)

	!temp = 0
	call min(lum, !HLSMAX, !temp)
	lum = !temp

	call min(hue, !HLSMAX, !temp)
	hue = !temp

	call min(sat, !HLSMAX, !temp)
	sat = !temp
	
	if (sat = 0) then            ' achromatic case
		R=G=B=(lum*!RGBMAX)/!HLSMAX
		if (hue <> !UNDEFINED)  then
			' ERROR
		endif
	else                     ' chromatic case
		' set up magic numbers
		if (lum <= (!HLSMAX/2)) then
			!Magic2 = (lum*(!HLSMAX + sat) + (!HLSMAX/2))/!HLSMAX
		else
			!Magic2 = lum + sat - ((lum*sat) + (!HLSMAX/2))/!HLSMAX
		endif
		!Magic1 = 2*lum-!Magic2

		' get RGB, change units from HLSMAX to RGBMAX
		call HueToRGB(!Magic1, !Magic2, hue+(!HLSMAX/3), !temp)		
		scalar RTrue = @round((!temp*!RGBMAX + (!HLSMAX/2))/!HLSMAX)
		call min(RTrue, !RGBMAX, R)
		
		call HueToRGB(!Magic1, !Magic2, hue, !temp)
		scalar GTrue = @round((!temp*!RGBMAX + (!HLSMAX/2)) / !HLSMAX)
		call min(GTrue, !RGBMAX, G)	
			
		call HueToRGB(!Magic1, !Magic2, hue-(!HLSMAX/3), !temp)
		scalar BTrue = @round((!temp*!RGBMAX + (!HLSMAX/2))/!HLSMAX)
		call min(BTrue, !RGBMAX, B)
	endif
endsub


